set linesize 32000
set pagesize 1000
set echo on
set termout on
set trimspool on

spool example.log

set define off

drop table sales;
drop table products;
drop table customers;

create table sales
(
sale_date date,
product_number number,
customer_number number,
amount number
);

create table products
(
product_number number,
product_type varchar2(12),
product_name varchar2(12)
);

create table customers
(
customer_number number,
customer_state varchar2(14),
customer_name varchar2(17)
);

insert into sales values (to_date('01/02/2012','MM/DD/YYYY'),1,1,100);
insert into sales values (to_date('01/03/2012','MM/DD/YYYY'),1,2,200);
insert into sales values (to_date('01/04/2012','MM/DD/YYYY'),2,1,300);
insert into sales values (to_date('01/05/2012','MM/DD/YYYY'),2,2,400);

insert into products values (1,'Cheese','Chedder');
insert into products values (2,'Cheese','Feta');

insert into customers values (1,'FL','Sunshine State Co');
insert into customers values (2,'FL','Green Valley Inc');

commit;

select 
sale_date, product_name, customer_name, amount
from sales, products, customers
where
sales.product_number=products.product_number and
sales.customer_number=customers.customer_number and
sale_date between 
  to_date('01/01/2012','MM/DD/YYYY') and 
  to_date('01/31/2012','MM/DD/YYYY') and
product_type = 'Cheese' and
customer_state = 'FL';

set autotrace on

-- joining products and customers
-- cartesian join

select 
product_name,customer_name
from products, customers
where
product_type = 'Cheese' and
customer_state = 'FL';

set autotrace off

-- selectivity of sales condition

-- # selected rows

select 
count(*) 
from sales
where
sale_date between 
  to_date('01/01/2012','MM/DD/YYYY') and 
  to_date('01/31/2012','MM/DD/YYYY');
  
-- total #rows
  
select 
count(*) 
from sales;

-- sales and products subtree

-- with non-joining where conditions

select count(*)
from sales, products
where
sales.product_number=products.product_number and
sale_date between 
  to_date('01/01/2012','MM/DD/YYYY') and 
  to_date('01/31/2012','MM/DD/YYYY') and
product_type = 'Cheese';

-- joining where condition only

select count(*)
from sales, products
where
sales.product_number=products.product_number;

-- example of gathering statistics setting estimate percent
-- and histogram on a column

-- 1 - set preferences

begin

DBMS_STATS.SET_TABLE_PREFS(NULL,'SALES','ESTIMATE_PERCENT','10');
DBMS_STATS.SET_TABLE_PREFS(NULL,'SALES','METHOD_OPT',
  'FOR COLUMNS SALE_DATE SIZE 254 PRODUCT_NUMBER SIZE 1 '||
  'CUSTOMER_NUMBER SIZE 1 AMOUNT SIZE 1');

end;
/

-- 2 - regather table stats with new preferences

execute DBMS_STATS.GATHER_TABLE_STATS (NULL,'SALES');

select last_analyzed,sample_size from user_tables where table_name='SALES';

-- Defaults for SET AUTOTRACE EXPLAIN report

COLUMN id_plus_exp FORMAT 990 HEADING i
COLUMN parent_id_plus_exp FORMAT 990 HEADING p
COLUMN plan_plus_exp FORMAT a2000
COLUMN object_node_plus_exp FORMAT a8
COLUMN other_tag_plus_exp FORMAT a29
COLUMN other_plus_exp FORMAT a44

-- cardinality hint

set autotrace on

select /*+cardinality(sales 1) */
sale_date, product_name, customer_name, amount
from sales, products, customers
where
sales.product_number=products.product_number and
sales.customer_number=customers.customer_number and
sale_date between 
  to_date('01/01/2012','MM/DD/YYYY') and 
  to_date('01/31/2012','MM/DD/YYYY') and
product_type = 'Cheese' and
customer_state = 'FL';

set autotrace off

-- leading hint

set autotrace on

select /*+leading(sales) */
sale_date, product_name, customer_name, amount
from sales, products, customers
where
sales.product_number=products.product_number and
sales.customer_number=customers.customer_number and
sale_date between 
  to_date('01/01/2012','MM/DD/YYYY') and 
  to_date('01/31/2012','MM/DD/YYYY') and
product_type = 'Cheese' and
customer_state = 'FL';

set autotrace off

-- break into pieces

drop table sales_product_results;

create global temporary table sales_product_results
(
sale_date date,
customer_number number,
amount number,
product_type varchar2(12),
product_name varchar2(12)
) on commit preserve rows;

insert /*+append */ 
into sales_product_results
select 
sale_date,
customer_number,
amount,
product_type,
product_name
from sales, products
where
sales.product_number=products.product_number and
sale_date between 
  to_date('01/01/2012','MM/DD/YYYY') and 
  to_date('01/31/2012','MM/DD/YYYY') and
product_type = 'Cheese';

commit;

select 
sale_date, product_name, customer_name, amount
from sales_product_results spr, customers c
where
spr.customer_number=c.customer_number and
c.customer_state = 'FL';

-- nested loops

create index products_index on products(product_number);
create index customers_index on customers(customer_number);

set autotrace on

select /*+ leading(sales products) use_nl(sales products customers)
use_nl(products customers) index(products) index(customers) */
sale_date, product_name, customer_name, amount
from sales, products, customers
where
sales.product_number=products.product_number and
sales.customer_number=customers.customer_number and
sale_date between 
  to_date('01/01/2012','MM/DD/YYYY') and 
  to_date('01/31/2012','MM/DD/YYYY') and
product_type = 'Cheese' and
customer_state = 'FL';

set autotrace off

-- mixture on nested loops and hash join

set autotrace on

select /*+ leading(sales products) use_hash(sales products)
use_nl(products customers) index(customers) full(products) */
sale_date, product_name, customer_name, amount
from sales, products, customers
where
sales.product_number=products.product_number and
sales.customer_number=customers.customer_number and
sale_date between 
  to_date('01/01/2012','MM/DD/YYYY') and 
  to_date('01/31/2012','MM/DD/YYYY') and
product_type = 'Cheese' and
customer_state = 'FL';

set autotrace off

-- parallel query example

alter table sales parallel 8;
alter table customers parallel 8;
alter table products parallel 8;

set autotrace on

select 
sale_date, product_name, customer_name, amount
from sales, products, customers
where
sales.product_number=products.product_number and
sales.customer_number=customers.customer_number and
sale_date between 
  to_date('01/01/2012','MM/DD/YYYY') and 
  to_date('01/31/2012','MM/DD/YYYY') and
product_type = 'Cheese' and
customer_state = 'FL';

set autotrace off

alter table sales noparallel;
alter table customers noparallel;
alter table products noparallel;

set autotrace on

-- hints to force full scan or index scan
-- access methods

select /*+ full(sales) index(customers) index(products) */
sale_date, product_name, customer_name, amount
from sales, products, customers
where
sales.product_number=products.product_number and
sales.customer_number=customers.customer_number and
sale_date between 
  to_date('01/01/2012','MM/DD/YYYY') and 
  to_date('01/31/2012','MM/DD/YYYY') and
product_type = 'Cheese' and
customer_state = 'FL';

set autotrace off

-- example of using timing

set timing on

select 
sale_date, product_name, customer_name, amount
from sales, products, customers
where
sales.product_number=products.product_number and
sales.customer_number=customers.customer_number and
sale_date between 
  to_date('01/01/2012','MM/DD/YYYY') and 
  to_date('01/31/2012','MM/DD/YYYY') and
product_type = 'Cheese' and
customer_state = 'FL';


spool off
